@@ -31,7 +31,7 @@ from nuitka.importing.Importing import (
 )
 from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.optimizations.TraceCollections import ConstraintCollectionModule
-from nuitka.SourceCodeReferences import SourceCodeReference
+from nuitka.SourceCodeReferences import SourceCodeReference, fromFilename
 from nuitka.utils import Utils
 
 from .Checkers import checkStatementsSequenceOrNone
@@ -85,7 +85,6 @@ class PythonModuleMixin:
         if self.package_name is not None and self.package is None:
             package_package, package_filename, _finding = findModule(
                 importing      = self,
-                source_ref     = self.getSourceReference(),
                 module_name    = self.package_name,
                 parent_package = None,
                 level          = 1,
@@ -156,7 +155,7 @@ class PythonModuleMixin:
             current = filename
 
             levels = full_name.count('.')
-            if self.isPythonPackage():
+            if self.isCompiledPythonPackage():
                 levels += 1
 
             for _i in range(levels):
@@ -166,15 +165,13 @@ class PythonModuleMixin:
             return result
 
 
-class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
-                   ClosureGiverNodeBase):
-    """ Module
+class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
+                           ClosureGiverNodeBase):
+    """ Compiled Python Module
 
-        The module is the only possible root of a tree. When there are many
-        modules they form a forest.
     """
 
-    kind = "PYTHON_MODULE"
+    kind = "COMPILED_PYTHON_MODULE"
 
     named_children = (
         "body",
@@ -198,7 +195,6 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
             package_name = package_name
         )
 
-
         ChildrenHavingMixin.__init__(
             self,
             values = {
@@ -225,7 +221,7 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
         }
 
     def asXml(self):
-        result = super(PythonModule, self).asXml()
+        result = super(CompiledPythonModule, self).asXml()
 
         for function_body in self.active_functions:
             result.append(function_body.asXml())
@@ -278,7 +274,7 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
     setBody = ChildrenHavingMixin.childSetter("body")
 
     @staticmethod
-    def isPythonModule():
+    def isCompiledPythonModule():
         return True
 
     def getParent(self):
@@ -430,6 +426,95 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
 
         return False
 
+
+class CompiledPythonPackage(CompiledPythonModule):
+    kind = "COMPILED_PYTHON_PACKAGE"
+
+    def __init__(self, name, package_name, source_ref):
+        assert name
+
+        CompiledPythonModule.__init__(
+            self,
+            name         = name,
+            package_name = package_name,
+            source_ref   = source_ref
+        )
+
+    def getOutputFilename(self):
+        return Utils.dirname(self.getFilename())
+
+
+def makeUncompiledPythonModule(module_name, filename, bytecode, is_package,
+                               user_provided):
+    parts = module_name.rsplit('.', 1)
+    name = parts[-1]
+
+    package_name = parts[0] if len(parts) == 2 else None
+    source_ref = fromFilename(filename)
+
+    if is_package:
+        return UncompiledPythonPackage(
+            name          = name,
+            package_name  = package_name,
+            bytecode      = bytecode,
+            filename      = filename,
+            user_provided = user_provided,
+            source_ref    = source_ref
+        )
+    else:
+        return UncompiledPythonModule(
+            name          = name,
+            package_name  = package_name,
+            bytecode      = bytecode,
+            filename      = filename,
+            user_provided = user_provided,
+            source_ref    = source_ref
+        )
+
+
+class UncompiledPythonModule(PythonModuleMixin, NodeBase):
+    """ Compiled Python Module
+
+    """
+
+    kind = "UNCOMPILED_PYTHON_MODULE"
+
+    def __init__(self, name, package_name, bytecode, filename, user_provided,
+                 source_ref):
+        NodeBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+        PythonModuleMixin.__init__(
+            self,
+            name         = name,
+            package_name = package_name
+        )
+
+        self.bytecode = bytecode
+        self.filename = filename
+
+        self.user_provided = user_provided
+
+    def isUserProvided(self):
+        return self.user_provided
+
+    def getByteCode(self):
+        return self.bytecode
+
+    @staticmethod
+    def isPackage():
+        return False
+
+class UncompiledPythonPackage(UncompiledPythonModule):
+    kind = "UNCOMPILED_PYTHON_PACKAGE"
+
+    @staticmethod
+    def isPackage():
+        return True
+
+
 class SingleCreationMixin:
     created = set()
 
@@ -438,11 +523,11 @@ class SingleCreationMixin:
         self.created.add(self.__class__)
 
 
-class PythonMainModule(PythonModule, SingleCreationMixin):
+class PythonMainModule(CompiledPythonModule, SingleCreationMixin):
     kind = "PYTHON_MAIN_MODULE"
 
     def __init__(self, main_added, source_ref):
-        PythonModule.__init__(
+        CompiledPythonModule.__init__(
             self,
             name         = "__main__",
             package_name = None,
@@ -461,14 +546,14 @@ class PythonMainModule(PythonModule, SingleCreationMixin):
         if self.main_added:
             return Utils.dirname(self.getFilename())
         else:
-            return PythonModule.getOutputFilename(self)
+            return CompiledPythonModule.getOutputFilename(self)
 
 
-class PythonInternalModule(PythonModule, SingleCreationMixin):
+class PythonInternalModule(CompiledPythonModule, SingleCreationMixin):
     kind = "PYTHON_INTERNAL_MODULE"
 
     def __init__(self):
-        PythonModule.__init__(
+        CompiledPythonModule.__init__(
             self,
             name         = "__internal__",
             package_name = None,
@@ -489,26 +574,11 @@ class PythonInternalModule(PythonModule, SingleCreationMixin):
         return "__internal"
 
 
-class PythonPackage(PythonModule):
-    kind = "PYTHON_PACKAGE"
-
-    def __init__(self, name, package_name, source_ref):
-        assert name
-
-        PythonModule.__init__(
-            self,
-            name         = name,
-            package_name = package_name,
-            source_ref   = source_ref
-        )
-
-    def getOutputFilename(self):
-        return Utils.dirname(self.getFilename())
-
-
 class PythonShlibModule(PythonModuleMixin, NodeBase):
     kind = "PYTHON_SHLIB_MODULE"
 
+    avoid_duplicates = set()
+
     def __init__(self, name, package_name, source_ref):
         NodeBase.__init__(
             self,
@@ -521,11 +591,17 @@ class PythonShlibModule(PythonModuleMixin, NodeBase):
             package_name = package_name
         )
 
+        # That would be a mistake we just made.
         assert Utils.basename(source_ref.getFilename()) != "<frozen>"
 
         # That is too likely a bug.
         assert name != "__main__"
 
+        # Duplicates should be avoided by us caching elsewhere before creating
+        # the object.
+        assert self.getFullName() not in self.avoid_duplicates, self.getFullName()
+        self.avoid_duplicates.add(self.getFullName())
+
     def getDetails(self):
         return {
             "name"         : self.name,
